@@ -23,35 +23,35 @@
 #include "ProgressReporter.h"
 #include "tgstat.h"
 
-unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweight, size_t num_edges,
-                       unsigned min_cluster_size, float cooling_rate, unsigned burn_in, unsigned *node2cluster, size_t num_nodes)
+unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweight, uint64_t num_edges,
+                       unsigned min_cluster_size, float cooling_rate, unsigned burn_in, unsigned *node2cluster, uint64_t num_nodes)
 {
     vdebug("Finding seeds...\n");
 
     unordered_map<pair<unsigned, unsigned>, float> ij2weight;
 
     ij2weight.reserve(num_edges);
-    for (size_t i = 0; i < num_edges; ++i)
+    for (uint64_t i = 0; i < num_edges; ++i)
         ij2weight[{pnode1[i] - 1, pnode2[i] - 1}] = pweight[i];
 
     vector<vector<unsigned>> incoming(num_nodes);
     vector<vector<unsigned>> outgoing(num_nodes);
-    vector<unsigned> node2seed(num_nodes, -1);
+    vector<unsigned> node2seed(num_nodes, (unsigned)-1);
     vector<unsigned> cluster_sizes;
     unsigned num_clusters = 0;
     unordered_set<unsigned> cands;                 // seed candidates 
     vector<unsigned> node2weight(num_nodes, 0);    // node2weight[i] = | incoming(i) - COVERED |
-    size_t weights_sum = 0;
+    uint64_t weights_sum = 0;
     unsigned max_weight = 0;
     unsigned min_weight = min_cluster_size;
 
-    for (size_t i = 0; i < num_edges; ++i) {
+    for (uint64_t i = 0; i < num_edges; ++i) {
         incoming[pnode2[i] - 1].push_back(pnode1[i] - 1);
         outgoing[pnode1[i] - 1].push_back(pnode2[i] - 1);
     }
 
     cands.reserve(num_nodes);
-    for (size_t i = 0; i < num_nodes; ++i) {
+    for (uint64_t i = 0; i < num_nodes; ++i) {
         node2weight[i] = incoming[i].size();
         max_weight = max(max_weight, node2weight[i]);
         if (node2weight[i] >= min_weight) {
@@ -81,7 +81,7 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
         cluster_sizes.push_back(0);
 
         for (auto i : incoming[seed]) {
-            if (node2seed[i] == -1) {
+            if (node2seed[i] == (unsigned)-1) {
                 for (auto j : outgoing[i]) {
                     --node2weight[j];
                     if (node2weight[j] < min_weight)
@@ -109,34 +109,25 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
         ++num_clusters;
     }
 
-//REprintf("Num clusters: %d\n", num_clusters);
-//for (auto i = node2seed.begin(); i < node2seed.end(); ++i)
-//REprintf("\tnode2seed %d -> %d\n", i - node2seed.begin(), *i);
-//REprintf("Num clusters: %d\n", num_clusters);
-//for (size_t i = 0; i < num_nodes; ++i)
-//REprintf("\t%d -> %d\n", i, node2cluster[i]);
-
     // look for the nodes that have not been covered yet and assign them to the seeds
     // that achieve the maximal weight along the d2 path; nodes that cannot reach the seeds within d2 will stay unassigned
     for (auto i = node2seed.begin(); i < node2seed.end(); ++i) {
-        if (*i == -1) {
+        if (*i == (unsigned)-1) {
             float best_weight = -1;
-            unsigned best_seed = -1;
-            unsigned best_cluster = -1;
+            unsigned best_seed = (unsigned)-1;
             unsigned origin = i - node2seed.begin();
 
             for (auto j : outgoing[origin]) {
-                if (node2seed[j] != -1 &&                  // j is covered
+                if (node2seed[j] != (unsigned)-1 &&        // j is covered
                     ij2weight.count({j, node2seed[j]}) &&  // j is connected directly to the seed (and not via someone else)
                     best_weight < ij2weight[{origin, j}] + ij2weight[{j, node2seed[j]}])    // i->j->seed achieves the highest weight
                 {
                     best_weight = ij2weight[{origin, j}] + ij2weight[{j, node2seed[j]}];
                     best_seed = node2seed[j];
-                    best_cluster = node2cluster[j];
                 }
             }
 
-            if (best_seed != -1) {
+            if (best_seed != (unsigned)-1) {
                 *i = best_seed;
                 node2cluster[i - node2seed.begin()] = node2cluster[best_seed];
                 ++cluster_sizes[node2cluster[best_seed]];
@@ -146,9 +137,6 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
 
     check_interrupt();
     vdebug("Formed %d clusters\n", num_clusters);
-//REprintf("Cluster sizes: %ld, num_clusters: %ld\n", cluster_sizes.size(), num_clusters);
-//for (auto i = node2seed.begin(); i < node2seed.end(); ++i)
-//REprintf("%d -> %d, cluster: %d, cluster size: %d\n", i - node2seed.begin(), *i, node2cluster[i - node2seed.begin()], node2cluster[i - node2seed.begin()] == -1 ? -1 : cluster_sizes[node2cluster[i - node2seed.begin()]]);
 
     // stage 2: consolidate the clusters
 
@@ -165,17 +153,17 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
     vector<float> cooling_rates(num_nodes, 1);
 
     vdebug("Starting consolidation of nodes around the clusters...\n");
-    for (size_t i = 0; i < num_nodes; ++i) {
+    for (uint64_t i = 0; i < num_nodes; ++i) {
         vector<Score> &cluster2score = node2cluster_score[i];
         cluster2score.resize(num_clusters);
 
         for (auto j : outgoing[i]) {
-            if (node2cluster[j] != -1)
+            if (node2cluster[j] != (unsigned)-1)
                 cluster2score[node2cluster[j]].votes_in += ij2weight[{i, j}];
         }
 
         for (auto j : incoming[i]) {
-            if (node2cluster[j] != -1)
+            if (node2cluster[j] != (unsigned)-1)
                 cluster2score[node2cluster[j]].votes_out += ij2weight[{j, i}];
         }
     }
@@ -183,7 +171,7 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
 
     vector<unsigned> candidates(num_nodes);
 
-    for (size_t i = 0; i < num_nodes; ++i)
+    for (uint64_t i = 0; i < num_nodes; ++i)
         candidates[i] = i;
 
     while (1) {
@@ -193,12 +181,12 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
 
         for (auto i : candidates) {
             // find the maximal score
-            int new_cluster = -1;
-            int old_cluster = node2cluster[i];
+            unsigned new_cluster = (unsigned)-1;
+            unsigned old_cluster = node2cluster[i];
             float max_score = 0;
 
             vector<Score> &cluster2score = node2cluster_score[i];
-            for (int cluster = 0; cluster < num_clusters; ++cluster) {
+            for (unsigned cluster = 0; cluster < num_clusters; ++cluster) {
                 const Score &cscore = cluster2score[cluster];
                 if (cscore.votes_in && cscore.votes_out) {
                     float score = cluster == old_cluster ?
@@ -212,13 +200,13 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
                 }
             }
 
-            if (new_cluster != -1 && new_cluster != old_cluster) {
+            if (new_cluster != (unsigned)-1 && new_cluster != old_cluster) {
                 // reassign the cluster
                 for (auto j : incoming[i]) {
                     float weight = ij2weight[{j, i}];
                     vector<Score> &cluster2score = node2cluster_score[j];
 
-                    if (old_cluster != -1)      // some nodes might not be assigned yet to any cluster
+                    if (old_cluster != (unsigned)-1)      // some nodes might not be assigned yet to any cluster
                         cluster2score[old_cluster].votes_in -= weight;
 
                     cluster2score[new_cluster].votes_in += weight;
@@ -228,7 +216,7 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
                     float weight = ij2weight[{i, j}];
                     vector<Score> &cluster2score = node2cluster_score[j];
 
-                    if (old_cluster != -1)
+                    if (old_cluster != (unsigned)-1)
                         cluster2score[old_cluster].votes_out -= weight;
 
                     cluster2score[new_cluster].votes_out += weight;
@@ -241,7 +229,7 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
                 if (reassignments[i] > burn_in)
                     cooling_rates[i] *= cooling_rate;
 
-                if (old_cluster != -1)
+                if (old_cluster != (unsigned)-1)
                     --cluster_sizes[old_cluster];
                 ++cluster_sizes[new_cluster];
             }
@@ -254,10 +242,6 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
     }
     vdebug("Consolidation - DONE\n");
 
-//for (unsigned i = 0; i < num_clusters; ++i) {
-//REprintf("Cluster[%d], size: %d\n", i, cluster_sizes[i]);
-//}
-//
     if (0 && g_tgstat->debug()) {
         double total_weight = 0;
         double in_cluster_weight = 0;
@@ -272,7 +256,7 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
             double weight = edge_weight.second;
 
             total_weight += weight;
-            if (node2cluster[i] == -1 || node2cluster[j] == -1)
+            if (node2cluster[i] == (unsigned)-1 || node2cluster[j] == (unsigned)-1)
                 unassigned_weight += weight;
             else {
                 if (node2cluster[i] == node2cluster[j]) {
@@ -300,26 +284,23 @@ unsigned graph2cluster(const int *pnode1, const int *pnode2, const double *pweig
         }
     }
 
-//for (unsigned i = 0; i < num_nodes; ++i)
-//REprintf("Node %d => %d\n", i + 1, node2cluster[i] + 1);
-
     return num_clusters;
 }
 
-void reassign_weights(const int *pnode1, const int *pnode2, unsigned knn, size_t num_edges,
+void reassign_weights(const int *pnode1, const int *pnode2, unsigned knn, uint64_t num_edges,
                       const vector<bool> &node_selected, vector<int> &nodes1, vector<int> &nodes2, vector<double> &weights)
 {
     unsigned last_idx = 0;
     unsigned last_node = pnode1[0] - 1;
     unsigned num_node_edges = node_selected[pnode2[0] - 1] ? 1 : 0;
 
-    for (size_t i = 1; ; ++i) {
-        if (i == num_edges || last_node != pnode1[i] - 1) {
+    for (uint64_t i = 1; ; ++i) {
+        if (i == num_edges || (int)last_node != pnode1[i] - 1) {
             if (node_selected[last_node] && num_node_edges) {
                 double rank = 0;
                 num_node_edges = min(num_node_edges, knn);
 
-                for (size_t j = last_idx; j < i; ++j) {
+                for (uint64_t j = last_idx; j < i; ++j) {
                     unsigned pointed_node = pnode2[j] - 1;
 
                     if (node_selected[pointed_node]) {
@@ -346,15 +327,15 @@ void reassign_weights(const int *pnode1, const int *pnode2, unsigned knn, size_t
     }
 }
 
-void launch_kid_hash(const int *pnode1, const int *pnode2, const double *pweight, size_t num_nodes, size_t num_edges, unsigned *res, int slot,
+void launch_kid_hash(const int *pnode1, const int *pnode2, const double *pweight, uint64_t num_nodes, uint64_t num_edges, unsigned *res, int slot,
                      unsigned knn, double p_resamp, unsigned min_cluster_size, float cooling_rate, unsigned burn_in, uint64_t seed)
 {
     vdebug("Launching a working process at slot %d\n", slot);
     if (!TGStat::launch_process()) {     // child process
-        size_t num_kid_nodes = max(1., p_resamp * num_nodes);
+        uint64_t num_kid_nodes = max(1., p_resamp * num_nodes);
         vector<bool> node_selected(num_nodes, false);
         vector<unsigned> nodes(num_nodes);
-        for (size_t i = 0; i < num_nodes; ++i)
+        for (uint64_t i = 0; i < num_nodes; ++i)
             nodes[i] = i;
 
         g_tgstat->rnd_seed(seed);
@@ -379,22 +360,22 @@ void launch_kid_hash(const int *pnode1, const int *pnode2, const double *pweight
         g_tgstat->rnd_seed(seed);   // this will make the results reproducible by tgs_graph_cover
         *num_clusters = graph2cluster(nodes1.data(), nodes2.data(), weights.data(), nodes1.size(), min_cluster_size, cooling_rate, burn_in, clusters, num_nodes);
 
-        for (size_t i = num_kid_nodes; i < num_nodes; ++i)
-            clusters[nodes[i]] = -2;                          // mark nodes that were not selected by -2
+        for (uint64_t i = num_kid_nodes; i < num_nodes; ++i)
+            clusters[nodes[i]] = (unsigned)-2;                          // mark nodes that were not selected by -2
 
         *pready = 1;
         rexit();
     }
 }
 
-void launch_kid_full(const int *pnode1, const int *pnode2, const double *pweight, size_t num_nodes, size_t num_edges, unsigned *pready, unsigned short *pco_clust,
+void launch_kid_full(const int *pnode1, const int *pnode2, const double *pweight, uint64_t num_nodes, uint64_t num_edges, unsigned *pready, unsigned short *pco_clust,
                      unsigned short *psamples, unsigned knn, double p_resamp, unsigned min_cluster_size, float cooling_rate, unsigned burn_in, uint64_t seed)
 {
     if (!TGStat::launch_process()) {     // child process
         unsigned num_kid_nodes = max(1., p_resamp * num_nodes);
         vector<bool> node_selected(num_nodes, false);
         vector<unsigned> nodes(num_nodes);
-        for (size_t i = 0; i < num_nodes; ++i)
+        for (uint64_t i = 0; i < num_nodes; ++i)
             nodes[i] = i;
 
         g_tgstat->rnd_seed(seed);
@@ -406,7 +387,7 @@ void launch_kid_full(const int *pnode1, const int *pnode2, const double *pweight
             __sync_add_and_fetch(psamples + nodes[i], 1);
         }
 
-        vector<unsigned> node2cluster(num_nodes, -1);
+        vector<unsigned> node2cluster(num_nodes, (unsigned)-1);
         vector<int> nodes1;
         vector<int> nodes2;
         vector<double> weights;
@@ -419,8 +400,8 @@ void launch_kid_full(const int *pnode1, const int *pnode2, const double *pweight
 
         vector<vector<unsigned>> cluster2nodes(num_clusters);
 
-        for (size_t i = 0; i < num_kid_nodes; ++i) {
-            if (node2cluster[nodes[i]] != -1)
+        for (uint64_t i = 0; i < num_kid_nodes; ++i) {
+            if (node2cluster[nodes[i]] != (unsigned)-1)
                 cluster2nodes[node2cluster[nodes[i]]].push_back(nodes[i]);
         }
 
@@ -428,7 +409,7 @@ void launch_kid_full(const int *pnode1, const int *pnode2, const double *pweight
             sort(cluster_nodes.begin(), cluster_nodes.end());
             for (auto inode1 = cluster_nodes.begin(); inode1 < cluster_nodes.end(); ++inode1) {
                 // this ugly offset translates node1, node2 to the index in the buffer that stores the lower triangle (incl. diagonal) of the co_cluster matrix
-                unsigned short *p = pco_clust + ((size_t)*inode1 * (2 * num_nodes - (size_t)*inode1 - 1)) / 2;
+                unsigned short *p = pco_clust + ((uint64_t)*inode1 * (2 * num_nodes - (uint64_t)*inode1 - 1)) / 2;
                 for (auto inode2 = inode1; inode2 < cluster_nodes.end(); ++inode2)
                     __sync_add_and_fetch(p + *inode2, 1);
             }
@@ -439,14 +420,14 @@ void launch_kid_full(const int *pnode1, const int *pnode2, const double *pweight
     }
 }
 
-void launch_kid_edges(const int *pnode1, const int *pnode2, const double *pweight, size_t num_nodes, size_t num_edges, unsigned *pready, unsigned short *pco_clust,
+void launch_kid_edges(const int *pnode1, const int *pnode2, const double *pweight, uint64_t num_nodes, uint64_t num_edges, unsigned *pready, unsigned short *pco_clust,
                       unsigned short *psamples, unsigned knn, double p_resamp, unsigned min_cluster_size, float cooling_rate, unsigned burn_in, uint64_t seed)
 {
     if (!TGStat::launch_process()) {     // child process
         unsigned num_kid_nodes = max(1., p_resamp * num_nodes);
         vector<bool> node_selected(num_nodes, false);
         vector<unsigned> nodes(num_nodes);
-        for (size_t i = 0; i < num_nodes; ++i)
+        for (uint64_t i = 0; i < num_nodes; ++i)
             nodes[i] = i;
 
         g_tgstat->rnd_seed(seed);
@@ -469,11 +450,11 @@ void launch_kid_edges(const int *pnode1, const int *pnode2, const double *pweigh
         g_tgstat->rnd_seed(seed);   // this will make the results reproducible by tgs_graph_cover
         graph2cluster(nodes1.data(), nodes2.data(), weights.data(), nodes1.size(), min_cluster_size, cooling_rate, burn_in, node2cluster.data(), num_nodes);
 
-        for (size_t i = 0; i < num_edges; ++i) {
+        for (uint64_t i = 0; i < num_edges; ++i) {
             int node1 = pnode1[i] - 1;
             int node2 = pnode2[i] - 1;
 
-            if (node_selected[node1] && node_selected[node2] && node2cluster[node1] != -1 && node2cluster[node1] == node2cluster[node2])
+            if (node_selected[node1] && node_selected[node2] && node2cluster[node1] != (unsigned)-1 && node2cluster[node1] == node2cluster[node2])
                 __sync_add_and_fetch(pco_clust + i, 1);
         }
 
@@ -492,7 +473,7 @@ SEXP tgs_graph2cluster(SEXP _graph, SEXP _min_cluster_size, SEXP _cooling, SEXP
         int *pcol1;
         int *pcol2;
         double *pweight;
-        size_t num_edges;
+        uint64_t num_edges;
 
         {
             enum { COL1, COL2, WEIGHT, NUM_COLS };
@@ -533,11 +514,11 @@ SEXP tgs_graph2cluster(SEXP _graph, SEXP _min_cluster_size, SEXP _cooling, SEXP
         float cooling_rate = asReal(_cooling);
         unsigned burn_in = asInteger(_burn_in);
 
-        size_t num_nodes = 0;
+        uint64_t num_nodes = 0;
         if (rlevels1 == R_NilValue) {
-            for (size_t i = 0; i < num_edges; ++i) {
-                num_nodes = max(num_nodes, (size_t)pcol1[i]);
-                num_nodes = max(num_nodes, (size_t)pcol2[i]);
+            for (uint64_t i = 0; i < num_edges; ++i) {
+                num_nodes = max(num_nodes, (uint64_t)pcol1[i]);
+                num_nodes = max(num_nodes, (uint64_t)pcol2[i]);
             }
         } else
             num_nodes = xlength(rlevels1);
@@ -562,7 +543,7 @@ SEXP tgs_graph2cluster(SEXP _graph, SEXP _min_cluster_size, SEXP _cooling, SEXP
         for (int i = 0; i < NUM_COLS; i++)
             SET_STRING_ELT(rcolnames, i, mkChar(COL_NAMES[i]));
 
-        for (size_t i = 0; i < num_nodes; ++i) {
+        for (uint64_t i = 0; i < num_nodes; ++i) {
             INTEGER(rnode)[i] = i + 1;
             INTEGER(rcluster)[i] = node2cluster[i] + 1;
             INTEGER(rrownames)[i] = i + 1;
@@ -596,7 +577,7 @@ SEXP tgs_graph2cluster_multi_hash(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
 {
     SEXP answer = R_NilValue;
     unsigned *res = (unsigned *)MAP_FAILED;
-    size_t res_sizeof = 0;
+    uint64_t res_sizeof = 0;
 
 	try {
         TGStat tgstat(_envir);
@@ -604,7 +585,7 @@ SEXP tgs_graph2cluster_multi_hash(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
         int *pcol1;
         int *pcol2;
         double *pweight;
-        size_t num_edges;
+        uint64_t num_edges;
 
         {
             enum { COL1, COL2, WEIGHT, NUM_COLS };
@@ -656,20 +637,19 @@ SEXP tgs_graph2cluster_multi_hash(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
         unsigned knn = asInteger(_knn);
         unsigned n_resamp = asInteger(_n_resamp);
         double p_resamp = asReal(_p_resamp);
-        size_t num_nodes = 0;
+        uint64_t num_nodes = 0;
 
         if (rlevels1 == R_NilValue) {
-            for (size_t i = 0; i < num_edges; ++i) {
-                num_nodes = max(num_nodes, (size_t)pcol1[i]);
-                num_nodes = max(num_nodes, (size_t)pcol2[i]);
+            for (uint64_t i = 0; i < num_edges; ++i) {
+                num_nodes = max(num_nodes, (uint64_t)pcol1[i]);
+                num_nodes = max(num_nodes, (uint64_t)pcol2[i]);
             }
         } else
             num_nodes = xlength(rlevels1);
 
         int max_num_kids = min((int)n_resamp, max(g_tgstat->num_processes() - 1, 1));
-//max_num_kids = 1;
-        int num_kids_launched = 0;
-        int num_kids_finished = 0;
+        unsigned num_kids_launched = 0;
+        unsigned num_kids_finished = 0;
 
         unordered_map<pair<unsigned, unsigned>, unsigned short> co_cluster;
         vector<unsigned short> co_cluster_diag(num_nodes, 0);
@@ -688,7 +668,7 @@ SEXP tgs_graph2cluster_multi_hash(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
         //    [child process j]
         //       ...
 
-        size_t kid_res_sizeof = sizeof(unsigned) + sizeof(unsigned) + sizeof(unsigned) * num_nodes;
+        uint64_t kid_res_sizeof = sizeof(unsigned) + sizeof(unsigned) + sizeof(unsigned) * num_nodes;
 
         res_sizeof = kid_res_sizeof * max_num_kids;
         res = (unsigned *)mmap(NULL, res_sizeof, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
@@ -721,13 +701,13 @@ SEXP tgs_graph2cluster_multi_hash(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
 
                     unsigned *num_clusters = pready + 1;
                     unsigned *clusters = num_clusters + 1;
-                    size_t num_pairs = 0;
+                    uint64_t num_pairs = 0;
                     vector<vector<unsigned>> cluster2nodes(*num_clusters);
 
-                    for (size_t i = 0; i < num_nodes; ++i) {
-                        if (clusters[i] != -2) {
+                    for (uint64_t i = 0; i < num_nodes; ++i) {
+                        if (clusters[i] != (unsigned)-2) {
                             ++node2sample_cnt[i];
-                            if (clusters[i] != -1) {
+                            if (clusters[i] != (unsigned)-1) {
                                 cluster2nodes[clusters[i]].push_back(i);
                                 co_cluster_diag[i]++;
                             }
@@ -820,7 +800,7 @@ SEXP tgs_graph2cluster_multi_hash(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
 
         rprotect(rsamples = RSaneAllocVector(INTSXP, num_nodes));
 
-        for (size_t i = 0; i < num_nodes; ++i)
+        for (uint64_t i = 0; i < num_nodes; ++i)
             INTEGER(rsamples)[i] = node2sample_cnt[i];
 
         if (rlevels1 != R_NilValue) {
@@ -848,7 +828,7 @@ SEXP tgs_graph2cluster_multi_hash(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
         SET_VECTOR_ELT(answer, 1, rsamples);
     } catch (TGLException &e) {
         if (!TGStat::is_kid() && res != (unsigned *)MAP_FAILED) {
-            munmap(res, res_sizeof);
+            munmap((char *)res, res_sizeof);  // needs to be char * for some versions of Solaris
             res = (unsigned *)MAP_FAILED;
         }
         rerror("%s", e.msg());
@@ -857,7 +837,7 @@ SEXP tgs_graph2cluster_multi_hash(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
     }
 
     if (!TGStat::is_kid() && res != (unsigned *)MAP_FAILED) {
-        munmap(res, res_sizeof);
+        munmap((char *)res, res_sizeof);  // needs to be char * for some versions of Solaris
         res = (unsigned *)MAP_FAILED;
     }
     rreturn(answer);
@@ -867,7 +847,7 @@ SEXP tgs_graph2cluster_multi_full(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
 {
     SEXP answer = R_NilValue;
     unsigned *res = (unsigned *)MAP_FAILED;
-    size_t res_sizeof = 0;
+    uint64_t res_sizeof = 0;
 
     try {
         TGStat tgstat(_envir);
@@ -875,7 +855,7 @@ SEXP tgs_graph2cluster_multi_full(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
         int *pcol1;
         int *pcol2;
         double *pweight;
-        size_t num_edges;
+        uint64_t num_edges;
 
         {
             enum { COL1, COL2, WEIGHT, NUM_COLS };
@@ -928,20 +908,19 @@ SEXP tgs_graph2cluster_multi_full(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
         unsigned n_resamp = asInteger(_n_resamp);
         double p_resamp = asReal(_p_resamp);
 
-        size_t num_nodes = 0;
+        uint64_t num_nodes = 0;
         if (rlevels1 == R_NilValue) {
-            for (size_t i = 0; i < num_edges; ++i) {
-                num_nodes = max(num_nodes, (size_t)pcol1[i]);
-                num_nodes = max(num_nodes, (size_t)pcol2[i]);
+            for (uint64_t i = 0; i < num_edges; ++i) {
+                num_nodes = max(num_nodes, (uint64_t)pcol1[i]);
+                num_nodes = max(num_nodes, (uint64_t)pcol2[i]);
             }
         } else
             num_nodes = xlength(rlevels1);
 
         int max_num_kids = min((int)n_resamp, max(g_tgstat->num_processes() - 1, 1));
-//max_num_kids = 2;
-        int num_kids_launched = 0;
-        int num_kids_finished = 0;
-        size_t co_clust_size = ((num_nodes + 1) * num_nodes) / 2;
+        unsigned num_kids_launched = 0;
+        unsigned num_kids_finished = 0;
+        uint64_t co_clust_size = ((num_nodes + 1) * num_nodes) / 2;
 
         vdebug("Allocating shared memory for results\n");
 
@@ -1011,9 +990,9 @@ SEXP tgs_graph2cluster_multi_full(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
         const char *COL_NAMES[NUM_COLS] = { "node1", "node2", "cnt" };
 
         SEXP rco_clust, rsamples, rnode1, rnode2, rcount, rrownames, rcolnames, rnames;
-        size_t co_clust_nz_size = 0;
+        uint64_t co_clust_nz_size = 0;
 
-        for (size_t i = 0; i < co_clust_size; ++i) {
+        for (uint64_t i = 0; i < co_clust_size; ++i) {
             if (pco_clust[i])
                 ++co_clust_nz_size;
         }
@@ -1028,11 +1007,11 @@ SEXP tgs_graph2cluster_multi_full(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
         rprotect(rrownames = RSaneAllocVector(INTSXP, co_clust_nz_size));
 
         {
-            size_t row = 0;
+            uint64_t row = 0;
 
-            for (size_t node1 = 0; node1 < num_nodes; ++node1) {
+            for (uint64_t node1 = 0; node1 < num_nodes; ++node1) {
                 unsigned short *p = pco_clust + (node1 * (2 * num_nodes - node1 - 1)) / 2;
-                for (size_t node2 = node1; node2 < num_nodes; ++node2) {
+                for (uint64_t node2 = node1; node2 < num_nodes; ++node2) {
                     if (p[node2]) {
                         INTEGER(rnode1)[row] = node1 + 1;
                         INTEGER(rnode2)[row] = node2 + 1;
@@ -1057,7 +1036,7 @@ SEXP tgs_graph2cluster_multi_full(SEXP _graph, SEXP _knn, SEXP _min_cluster_size
 
         rprotect(rsamples = RSaneAllocVector(INTSXP, num_nodes));
 
-        for (size_t i = 0; i < num_nodes; ++i)
+        for (uint64_t i = 0; i < num_nodes; ++i)
             INTEGER(rsamples)[i] = psamples[i];
 
         if (rlevels1 != R_NilValue) {
@@ -1096,7 +1075,7 @@ SEXP tgs_graph2cluster_multi_edges(SEXP _graph, SEXP _knn, SEXP _min_cluster_siz
 {
     SEXP answer = R_NilValue;
     unsigned *res = (unsigned *)MAP_FAILED;
-    size_t res_sizeof = 0;
+    uint64_t res_sizeof = 0;
 
     try {
         TGStat tgstat(_envir);
@@ -1104,7 +1083,7 @@ SEXP tgs_graph2cluster_multi_edges(SEXP _graph, SEXP _knn, SEXP _min_cluster_siz
         int *pcol1;
         int *pcol2;
         double *pweight;
-        size_t num_edges;
+        uint64_t num_edges;
 
         {
             enum { COL1, COL2, WEIGHT, NUM_COLS };
@@ -1157,19 +1136,18 @@ SEXP tgs_graph2cluster_multi_edges(SEXP _graph, SEXP _knn, SEXP _min_cluster_siz
         unsigned n_resamp = asInteger(_n_resamp);
         double p_resamp = asReal(_p_resamp);
 
-        size_t num_nodes = 0;
+        uint64_t num_nodes = 0;
         if (rlevels1 == R_NilValue) {
-            for (size_t i = 0; i < num_edges; ++i) {
-                num_nodes = max(num_nodes, (size_t)pcol1[i]);
-                num_nodes = max(num_nodes, (size_t)pcol2[i]);
+            for (uint64_t i = 0; i < num_edges; ++i) {
+                num_nodes = max(num_nodes, (uint64_t)pcol1[i]);
+                num_nodes = max(num_nodes, (uint64_t)pcol2[i]);
             }
         } else
             num_nodes = xlength(rlevels1);
 
         int max_num_kids = min((int)n_resamp, max(g_tgstat->num_processes() - 1, 1));
-//max_num_kids = 1;
-        int num_kids_launched = 0;
-        int num_kids_finished = 0;
+        unsigned num_kids_launched = 0;
+        unsigned num_kids_finished = 0;
 
         vdebug("Allocating shared memory for results\n");
 
@@ -1242,9 +1220,9 @@ SEXP tgs_graph2cluster_multi_edges(SEXP _graph, SEXP _knn, SEXP _min_cluster_siz
 
         rprotect(answer = RSaneAllocVector(VECSXP, 2));
 
-        size_t co_clust_nz_size = 0;
+        uint64_t co_clust_nz_size = 0;
 
-        for (size_t i = 0; i < num_edges; ++i) {
+        for (uint64_t i = 0; i < num_edges; ++i) {
             if (pco_clust[i])
                 ++co_clust_nz_size;
         }
@@ -1259,9 +1237,9 @@ SEXP tgs_graph2cluster_multi_edges(SEXP _graph, SEXP _knn, SEXP _min_cluster_siz
         rprotect(rrownames = RSaneAllocVector(INTSXP, co_clust_nz_size));
 
         {
-            size_t row = 0;
+            uint64_t row = 0;
 
-            for (size_t i = 0; i < num_edges; ++i) {
+            for (uint64_t i = 0; i < num_edges; ++i) {
                 if (pco_clust[i]) {
                     INTEGER(rnode1)[row] = pcol1[i];
                     INTEGER(rnode2)[row] = pcol2[i];
@@ -1285,7 +1263,7 @@ SEXP tgs_graph2cluster_multi_edges(SEXP _graph, SEXP _knn, SEXP _min_cluster_siz
 
         rprotect(rsamples = RSaneAllocVector(INTSXP, num_nodes));
 
-        for (size_t i = 0; i < num_nodes; ++i)
+        for (uint64_t i = 0; i < num_nodes; ++i)
             INTEGER(rsamples)[i] = psamples[i];
 
         if (rlevels1 != R_NilValue) {
